@@ -6,6 +6,7 @@ Advanced Interface Switcher
 Abstract interface object
 '''
 
+
 from __future__ import with_statement
 import gtk, hildon, dbus, threading
 from advpowcommon.ui.Banners import HildonBanner
@@ -16,6 +17,8 @@ import advifswlib.util.cacher as cacher
 import advifswlib.util.config as config
 import advifswlib.util.locale as locale
 
+__all__ = ['InterfaceState', 'AbstractInterface']
+
 class InterfaceState():
   off, on, connected, connecting, disconnecting = range(5)
 
@@ -52,10 +55,11 @@ class AbstractInterface():
     self.__button.tap_and_hold_setup(None, None, 0)
     self.__button.connect('tap-and-hold', self.__button_tap_and_hold)
     self.__button.connect('destroy', self.__destroy)
+    self.__switch_dialog = None
     self.__smi_cacher = cacher.get()
-    self.__smi_animator = DITPBCachedAnimator([], 
-                                              self.__button.get_image().set_from_pixbuf, 
-                                              cacher=self.__smi_cacher, 
+    self.__smi_animator = DITPBCachedAnimator([],
+                                              self.__button.get_image().set_from_pixbuf,
+                                              cacher=self.__smi_cacher,
                                               time_out=250)
     self.state_control = None
     self.mode_control = None
@@ -77,7 +81,7 @@ class AbstractInterface():
 
     self.__running = True
     #print self.__name, self.state['value'], self.state['mode'], self.state['strength']
-    #self.__show_switch_dialog()
+    #gobject.timeout_add(10000, self.__show_switch_dialog)
     #self.__show_switch_dialog()
 
   def pre_init(self):
@@ -121,6 +125,8 @@ class AbstractInterface():
 
   def __destroy(self, button=None):
     if hasattr(self, 'state'):
+      if self.__switch_dialog:
+        self.__switch_dialog.destroy()
       self.__destroy_signals()
       self.__destroy_interfaces()
       #print '%s (%s): destroyed' % (self.__name, self.__class__)
@@ -128,42 +134,40 @@ class AbstractInterface():
 
   def __init_signals(self):
     for key in self.state['dbus']:
-      self.__init_signal(key)
+      self.init_signal(key)
 
-  def __init_signal(self, x):
+  def init_signal(self, x, force=False):
     try:
-      def callback(*args, **kwargs): self.__received_signal(x, *args, **kwargs)
-      setattr(self, 'received_%s' % x, callback)
-      self.state['dbus'][x]['match'] = self.__bus\
-                                       .add_signal_receiver(getattr(self, 'received_%s' % x), 
-                                                            signal_name=self.state['dbus'][x]['name'], 
-                                                            dbus_interface=self.state['dbus'][x]['iface'], 
-                                                            path=self.state['dbus'][x]['path'], 
-                                                            **self.state['dbus'][x]['args'])
-      return True
+      if self.get_config_value('show-signal-%s' % x) != False:
+        if not 'match' in self.state['dbus'][x] or force:
+          def callback(*args, **kwargs):
+            self.__received_signal(x, *args, **kwargs)
+          setattr(self, 'received_%s' % x, callback)
+          self.state['dbus'][x]['match'] = self.__bus\
+            .add_signal_receiver(getattr(self, 'received_%s' % x),
+                                 **self.state['dbus'][x])
+          #print '%s | signal \'%s\' created' % (self.__name, x)
+          return True
     except:
-      return False
+      print '%s | signal \'%s\' failed' % (self.__name, x)
+    return False
 
   def reinit_signal(self, x):
-    self.__destroy_signal(x)
-    self.__init_signal(x)
+    self.destroy_signal(x)
+    self.init_signal(x)
 
   def __destroy_signals(self):
     for key in self.state['dbus']:
-      self.__destroy_signal(key)
+      self.destroy_signal(key)
 
-  def __destroy_signal(self, x):
+  def destroy_signal(self, x):
     if 'match' in self.state['dbus'][x]:
       receiver_match = self.state['dbus'][x].pop('match')
       #self.__bus.clean_up_signal_match(receiver_match)
-      self.__bus.remove_signal_receiver(receiver_match,
-                                        signal_name=self.state['dbus'][x]['name'], 
-                                        dbus_interface=self.state['dbus'][x]['iface'], 
-                                        path=self.state['dbus'][x]['path']
-                                       )
+      self.__bus.remove_signal_receiver(receiver_match,**self.state['dbus'][x])
       receiver_match.remove()
       delattr(self, 'received_%s' % x)
-      #print '%s: destroyed' % (x)
+      #print '%s | signal \'%s\' destroyed' % (self.__name, x)
       return True
     return False
 
@@ -219,9 +223,9 @@ class AbstractInterface():
         if loaded:
           kwargs = {'introspect': self.interfaces[x][4],
                     'follow_name_owner_changes': True}
-          self.__interfaces[x] = dbus.Interface(self.get_bus()\
-                                                  .get_object(*self.interfaces[x][:2],**kwargs),
-                                                self.interfaces[x][2])
+          self.__interfaces[x] = dbus.Interface(\
+            self.get_bus().get_object(*self.interfaces[x][:2],**kwargs),
+            self.interfaces[x][2])
         return loaded
     except Exception, e:
       if x in self.__interfaces:
@@ -287,45 +291,57 @@ class AbstractInterface():
     return self.__switch(not(self.get_config_value('show-state-dialog')))
 
   def __switch(self, show_dialog):
-    self.start_animation()
+    self.__button.get_toplevel().hide()
     if show_dialog:
       gobject.idle_add(self.__show_switch_dialog)
     else:
+      self.start_animation()
       self.__switch_state(int(self.state['value'] == InterfaceState.off))
-    self.__button.get_toplevel().hide()
     return True
 
   def __show_switch_dialog(self):
-    settings_dialog = gtk.Dialog('%s' % (self.__name), None,#self.__button.window, 
-                                 gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR, 
+    if not self.__switch_dialog:
+      self.__switch_dialog = gtk.Dialog('%s' % (self.__name), None,#self.__button.window, 
+                                 gtk.DIALOG_NO_SEPARATOR,
                                  (gtk.STOCK_PREFERENCES, gtk.RESPONSE_OK))
-    self.state_control = ToggleControlMaemo5(self.__get_state_controls())
+      #gtk.DIALOG_MODAL | 
+      #gtk.DIALOG_DESTROY_WITH_PARENT | 
+      self.state_control = ToggleControlMaemo5(self.__get_state_controls())
+      self.state_control.set_active(self.__get_state_control_value())
+      self.__switch_dialog.vbox.pack_start(self.state_control.get_box(), True)
+      height = self.state_control.get_height()
 
-    self.state_control.set_active(self.__get_state_control_value())
-    settings_dialog.vbox.pack_start(self.state_control.get_box(), True)
-    height = self.state_control.get_height()
+      if len(self.state['modes']) > 1:
+        self.mode_control = ToggleControlMaemo5(self.__get_mode_controls())
+        self.mode_control.set_active(self.state['mode'])
+        self.__switch_dialog.vbox.pack_start(self.mode_control.get_box(), True)
+        height += self.mode_control.get_height()
 
-    if len(self.state['modes']) > 1:
-      self.mode_control = ToggleControlMaemo5(self.__get_mode_controls())
-      self.mode_control.set_active(self.state['mode'])
-      settings_dialog.vbox.pack_start(self.mode_control.get_box(), True)
-      height += self.mode_control.get_height()
-
-    settings_button = settings_dialog.action_area.get_children()[0]
-    settings_button.connect('clicked', self.__show_settings_dialog)
+      settings_button = self.__switch_dialog.action_area.get_children()[0]
+      settings_button.connect('clicked', self.__settings_button_clicked)
 
     #settings_dialog.vbox.set_size_request(-1, min(350, height))
-    settings_dialog.show_all()
+    self.__switch_dialog.show_all()
     #self.__processing.release()
-    response = settings_dialog.run()
-    if response == gtk.RESPONSE_DELETE_EVENT:
+    response = self.__switch_dialog.run()
+    #if response == gtk.RESPONSE_DELETE_EVENT:
+      #self.__switch_dialog.hide()
       #settings_dialog.destroy()
-      self.state_control.destroy()
-      self.state_control = None
-      if self.mode_control:
-        self.mode_control.destroy()
-        self.mode_control = None
-      settings_dialog.destroy()
+      #self.state_control.destroy()
+      #self.state_control = None
+      #if self.mode_control:
+        #self.mode_control.destroy()
+        #self.mode_control = None
+    #settings_dialog.destroy()
+    self.__switch_dialog.hide()
+#    if response == gtk.RESPONSE_OK:
+#      gobject.idle_add(self.__show_switch_dialog)
+    return False
+
+  def __settings_button_clicked(self, button=None, user_data=None):
+    self.__switch_dialog.hide()
+    self.__show_settings_dialog()
+    self.__show_switch_dialog()
 
   def __get_state_control_value(self):
     value = self.state['value']
@@ -420,10 +436,10 @@ class AbstractInterface():
 
   def __set_sai(self, key):
     if any(self.state['show']):
-      if self.state['show'][self.state['value']] and self.get_config_value('show-sa-icon'):
-        self.__set_status_area_icon(self.__name, key)
-      else:
-        self.__set_status_area_icon(self.__name, None, False)
+     self.__set_status_area_icon(self.__name,
+       *([key]
+       if self.state['show'][self.state['value']] and self.get_config_value('show-sa-icon')
+       else [None, False]))
 
   def __show_banner(self):
     windex = self.state['value']
@@ -435,5 +451,11 @@ class AbstractInterface():
                else self.__lcl.get(self.state['text'][windex]))
       if len(self.state['modes']) > 1:
         text = '%s (%s)' % \
-               (text, self.__lcl.get_item(self.state['modes'][self.state['mode']]))
+               (text,
+                self.__lcl.get_item(self.state['modes'][self.state['mode']]))
       HildonBanner.show_information(self.__button, text)
+
+  def config_changed(self, key, old_value, new_value):
+    cb = 'config_changed_%s' % key.replace('-','_')
+    if hasattr(self, cb):
+      getattr(self, cb)(old_value, new_value)
