@@ -13,6 +13,8 @@ class GSMInterface(AbstractInterface):
 
 #  radio_access_signal = 'radio_access_technology_change'
 
+  name = 'GSM'
+
   ssc_status = [('offline', 'inactive',
                  'simless', 'tablet_state',
                  'blocked_sim', 'sim_locked'),
@@ -40,23 +42,19 @@ class GSMInterface(AbstractInterface):
                                'Phone.Sim',
                                None,
                                False]}
-    self.state = {'name': 'GSM',
-                  'dbus': {'on': {'name': 'cellular_system_state_change',
-                                  'iface': self.interfaces['Phone'][2],
-                                  'path': self.interfaces['Phone'][1],
-                                  'args': {}},
-                           'connected': {'name': 'modem_state_changed_ind',
-                                         'iface': self.interfaces['SSC'][2],
-                                         'path': self.interfaces['SSC'][1],
-                                         'args': {}},
-                           'strength': {'name': 'signal_strength_change',
-                                        'iface': self.interfaces['Phone'][2],
-                                        'path': self.interfaces['Phone'][1],
-                                        'args': {}},
-                           'operator': {'name': 'operator_name_change',
-                                        'iface': self.interfaces['Phone'][2],
-                                        'path': self.interfaces['Phone'][1],
-                                        'args': {}},
+    self.state = {'name': self.name,
+                  'dbus': {'on': {'signal_name': 'cellular_system_state_change',
+                                  'dbus_interface': self.interfaces['Phone'][2],
+                                  'path': self.interfaces['Phone'][1]},
+                           'connected': {'signal_name': 'modem_state_changed_ind',
+                                         'dbus_interface': self.interfaces['SSC'][2],
+                                         'path': self.interfaces['SSC'][1]},
+                           'strength': {'signal_name': 'signal_strength_change',
+                                        'dbus_interface': self.interfaces['Phone'][2],
+                                        'path': self.interfaces['Phone'][1]},
+                           'operator': {'signal_name': 'operator_name_change',
+                                        'dbus_interface': self.interfaces['Phone'][2],
+                                        'path': self.interfaces['Phone'][1]},
                           },
                   'images': ('ifsw_statusarea_cell_off',
                              'ifsw_statusarea_cell_signal_0',
@@ -93,6 +91,7 @@ class GSMInterface(AbstractInterface):
         result = InterfaceState.connected
         if self.get_config_value('show-signal-strength'):
           strength = self.get_strength()
+        #if self.get_config_value('show-network-in-sm'):
         sp_info = self.get_interface('SIM').get_service_provider_name()
         if sp_info:
           self.process_signal_operator(None, sp_info[0])
@@ -135,10 +134,12 @@ class GSMInterface(AbstractInterface):
   def run_commands(self, flag):
     self.get_interface('SSC').set_radio(flag)
     self.switched()
+    return False
 
   def run_mode_commands(self, mode):
     self.get_interface('Phone').set_selected_radio_access_technology(dbus.Byte(mode))
     self.switched()
+    return False
 
   def get_mode(self):
     return int(self.get_interface('Phone').get_selected_radio_access_technology()[0])
@@ -149,3 +150,10 @@ class GSMInterface(AbstractInterface):
 
   def get_strength_value(self, value):
     return (value+20)/21 #int(round(min(1.0, float(value)/100.0)*5))
+
+  def config_changed_show_signal_strength(self, old_value, new_value):
+    if new_value:
+      self.init_signal('strength')
+      self.state['strength'] = self.get_strength()
+    else:
+      self.destroy_signal('strength')
