@@ -13,6 +13,8 @@ from advpowcommon.util.execn import *
 
 class WLANInterface(AbstractInterface):
 
+  name = 'WLAN'
+
   #connection changed
   #0 DBUS_TYPE_STRING              service type or empty string
   #1 DBUS_TYPE_UINT32              service attributes
@@ -56,7 +58,7 @@ class WLANInterface(AbstractInterface):
               'load0': 'sudo /usr/bin/advifsw/wlan_load0.sh',
               'load1': 'sudo /usr/bin/advifsw/wlan_load1.sh',
               'get_driver': "lsmod | grep '^wl12\([x]\{2\}\|51\)\ ' | sed -n s/\ .*//p",
-              'check_be': 'if [ -d /opt/lib/modules/`uname -r` ]; then echo 1; fi'
+              'check_be': 'sudo /usr/bin/advifsw/get_wlan_path.sh'
              }
   # off: down > unload
   wifi_turn_off_cmd = [wifi_cmd['down'], wifi_cmd['unload']]
@@ -90,19 +92,20 @@ class WLANInterface(AbstractInterface):
                                         None,
                                         False]}
 
-    self.state = {'name': 'WLAN',
-                  'dbus': {'on': {'name': None,
-                                  'iface': self.interfaces['Manager'][2],
-                                  'path': self.interfaces['Manager'][1],
-                                  'args': {'member_keyword': 'member'}},
-                           'state': {'name': 'state_sig',
-                                     'iface': self.interfaces['ICD2'][2],
-                                     'path': self.interfaces['ICD2'][1],
-                                     'args': {}},
-                           'statistics': {'name': 'statistics_sig',
-                                          'iface': self.interfaces['ICD2'][2],
-                                          'path': self.interfaces['ICD2'][1],
-                                          'args': {}}},
+    self.state = {'name': self.name,
+                  'dbus': {'on': {
+                             'signal_name': None,
+                             'dbus_interface': self.interfaces['Manager'][2],
+                             'path': self.interfaces['Manager'][1],
+                             'member_keyword': 'member'},
+                           'connected': {
+                             'signal_name': 'state_sig',
+                             'dbus_interface': self.interfaces['ICD2'][2],
+                             'path': self.interfaces['ICD2'][1]},
+                           'strength': {
+                             'signal_name': 'statistics_sig',
+                             'dbus_interface': self.interfaces['ICD2'][2],
+                             'path': self.interfaces['ICD2'][1]}},
                   'images': ('ifsw_statusarea_wlan_off', 
                              'ifsw_statusarea_wlan_signal_0', 
                              'ifsw_statusarea_wlan_signal_3',
@@ -144,7 +147,7 @@ class WLANInterface(AbstractInterface):
         self.__cancel_get_strength()
     return result, mode, None
 
-  def process_signal_state(self, *args):
+  def process_signal_connected(self, *args):
     result = None
     if len(args) == 8 and args[3].startswith('WLAN_'):
       if args[7] == 2:
@@ -174,7 +177,7 @@ class WLANInterface(AbstractInterface):
     #print 'args:', args
     return result, None, None
 
-  def process_signal_statistics(self, *args):
+  def process_signal_strength(self, *args):
     return None, None, min(int(args[7]), 5)
 
   def get_status(self):
@@ -212,6 +215,7 @@ class WLANInterface(AbstractInterface):
     with self.__lock:
       run_shell_command(' && '.join(cmds))
       self.switched()
+    return False
 
   def show_connection_dialog(self):
     try:
@@ -219,6 +223,7 @@ class WLANInterface(AbstractInterface):
     except:
       self.reinit_interface('ICD_UI')
       self.get_interface('ICD_UI').show_conn_dlg(False)
+    return False
 
   def scan_request(self):
     try:
@@ -276,3 +281,10 @@ class WLANInterface(AbstractInterface):
         self.get_interface('ICD2').disconnect_req(dbus.UInt32(0x8000))
       except:
         self.reinit_interface('ICD2')
+
+  def config_changed_show_signal_strength(self, old_value, new_value):
+    if new_value:
+      self.init_signal('strength')
+      self.get_strength()
+    else:
+      self.destroy_signal('strength')
