@@ -6,7 +6,8 @@ path=$1
 if [ -d $path ]; then
   for file in $(ls $path); do
     if [ ! -d $path/$file ] && [ -x $path/$file ]; then
-      sh $path/$file
+      #echo $path/$file
+      $path/$file
     fi
   done
 fi
